@@ -31,6 +31,7 @@ import sys, re, os, time, subprocess
 # Our testing module
 import svntest
 from svntest import wc, actions
+import logging
 
 # (abbreviation)
 Skip = svntest.testcase.Skip_deco
@@ -41,6 +42,8 @@ Issue = svntest.testcase.Issue_deco
 Wimp = svntest.testcase.Wimp_deco
 Item = wc.StateItem
 
+logger = logging.getLogger()
+
 #----------------------------------------------------------------------
 # Helper function for testing stderr from co.
 # If none of the strings in STDERR list matches the regular expression
@@ -50,10 +53,9 @@ def test_stderr(re_string, stderr):
   for line in stderr:
     if exp_err_re.search(line):
       return
-  if svntest.main.options.verbose:
-    for x in stderr:
-      sys.stdout.write(x)
-    print("Expected stderr reg-ex: '" + re_string + "'")
+  for x in stderr:
+    logger.debug(x[:-1])
+  logger.info("Expected stderr reg-ex: '" + re_string + "'")
   raise svntest.Failure("Checkout failed but not in the expected way")
 
 #----------------------------------------------------------------------
@@ -1050,7 +1052,7 @@ def checkout_wc_from_drive(sbox):
   svntest.main.safe_rmtree(sbox.wc_dir)
   os.mkdir(sbox.wc_dir)
 
-  # create a virtual drive to the repository folder
+  # create a virtual drive to the working copy folder
   drive = find_the_next_available_drive_letter()
   if drive is None:
     raise svntest.Skip
@@ -1086,49 +1088,8 @@ def checkout_wc_from_drive(sbox):
     })
     svntest.actions.run_and_verify_checkout(repo_url, wc_dir,
                                             expected_output, expected_wc,
-                                            None, None, None, None)
-
-    wc2_dir = sbox.add_wc_path('2')
-    expected_output = wc.State(wc2_dir, {
-      'D'                 : Item(status='A '),
-      'D/H'               : Item(status='A '),
-      'D/H/psi'           : Item(status='A '),
-      'D/H/chi'           : Item(status='A '),
-      'D/H/omega'         : Item(status='A '),
-      'D/G'               : Item(status='A '),
-      'D/G/tau'           : Item(status='A '),
-      'D/G/pi'            : Item(status='A '),
-      'D/G/rho'           : Item(status='A '),
-      'D/gamma'           : Item(status='A '),
-      'C'                 : Item(status='A '),
-      'mu'                : Item(status='A '),
-      'B'                 : Item(status='A '),
-      'B/E'               : Item(status='A '),
-      'B/E/alpha'         : Item(status='A '),
-      'B/E/beta'          : Item(status='A '),
-      'B/F'               : Item(status='A '),
-      'B/lambda'          : Item(status='A '),
-    })
-    svntest.actions.run_and_verify_checkout(repo_url + '/A', wc2_dir,
-                                            expected_output, None,
-                                            None, None, None, None)
-
-    wc3_dir = sbox.add_wc_path('3')
-    expected_output = wc.State(wc3_dir, {
-      'H'                 : Item(status='A '),
-      'H/psi'             : Item(status='A '),
-      'H/chi'             : Item(status='A '),
-      'H/omega'           : Item(status='A '),
-      'G'                 : Item(status='A '),
-      'G/tau'             : Item(status='A '),
-      'G/pi'              : Item(status='A '),
-      'G/rho'             : Item(status='A '),
-      'gamma'             : Item(status='A '),
-    })
-
-    svntest.actions.run_and_verify_checkout(repo_url + '/A/D', wc3_dir,
-                                            expected_output, None,
-                                            None, None, None, None)
+                                            None, None, None, None,
+                                            '--force')
 
   finally:
     os.chdir(was_cwd)
